#include "hoops_license.h"
#include "HDB.h"
#include "HBaseModel.h"
#include "HBaseView.h"
#include "HIOManager.h"
#include "PointCloudAPI.h"
#include <thread>
#include <vector>


struct ExitCode {
	enum {
		Success = 0,
		Bad_Command_Line_Arguments,
		Could_Not_Load_File,
		OOC_Failure,
	};
};


static ooc::Env load_ooc (HBaseView & view, char const * file_name)
{
	wchar_t w_file_name[4096];
	std::copy_n(file_name, strlen(file_name) + 1, w_file_name);

	HC_KEY destination = view.GetModelKey();

	HInputHandlerOptions options;
	options.m_pHBaseView = &view;
	options.m_bOOCAutoUpdate = false; // Manually control rendering of OOC data for demo code.

	ooc::io::IOResult result = ooc::io::FileInputByKey(w_file_name, destination, options);
	if (result != ooc::io::IOResult_Success) {
		return ooc::Env(); // Return an invalid Env.
	}

	HC_KEY ooc_root = HC_Create_Segment_Key_By_Key(destination, "root");
	ooc::Env env;
	ooc::GetEnv(ooc_root, env);
	return env;
}


static void update (HBaseView & view, ooc::Env env)
{
	view.Update(); // Trigger OOC nodes to start populating.

	while (ooc::PollNodesLoadingOrHaveBeenLoaded(env)) {
		std::this_thread::sleep_for(std::chrono::milliseconds(50));
		view.Update(); // Render any newly populated OOC nodes.
	}
}


class CollectAllFilter : public ooc::query::Filter {
	virtual bool RejectPointsInMemory () {
		return false;
	}

	virtual bool RejectPointsOnDisk () {
		return false;
	}

	virtual bool RejectNode (ooc::NodeHandle const & node_handle) {
		UNREFERENCED(node_handle);
		return false;
	}

	virtual bool RejectBounding (ooc::Point const & min, ooc::Point const & max) {
		UNREFERENCED(min);
		UNREFERENCED(max);
		return false;
	}

	virtual bool AcceptPoint (ooc::Point const & point, size_t point_index) {
		UNREFERENCED(point);
		UNREFERENCED(point_index);
		return true;
	}
};


int main (int argc, char ** args)
{
	if (argc != 2) {
		return ExitCode::Bad_Command_Line_Arguments;
	}

	HC_Define_System_Options("license = `" HOOPS_LICENSE "`");
	HC_Define_System_Options("multi-threading = full"); // Multi-threading needs to be enabled for asynchronous (default) OOC use.

	HDB hdb;
	hdb.Init();
	HBaseModel model;
	model.Init();
	HBaseView view(&model);
	view.Init();

	char const * file_name = args[1];
	ooc::Env env = load_ooc(view, file_name);
	if (env == ooc::Env()) {
		return ExitCode::Could_Not_Load_File;
	}

	ooc::Point sum_of_all_coordinates;
	size_t num_points = 0;
	CollectAllFilter filter;
	ooc::query::QueryIterator it = ooc::query::QueryPoints(env, filter);
	while (true) {
		ooc::query::QueryIterator::Status status = it.GetStatus();
		if (status == ooc::query::QueryIterator::Status_Dead) {
			break; // No more points to iterate.
		}
		if (status != ooc::query::QueryIterator::Status_Alive) {
			return ExitCode::OOC_Failure;
		}
		sum_of_all_coordinates = sum_of_all_coordinates + it->GetNodePoint();
		++num_points;
		it.Advance();
	}

	HC_Open_Segment_By_Key(view.GetModelKey());{
		ooc::Point average_coordinate = sum_of_all_coordinates;
		average_coordinate.x /= num_points;
		average_coordinate.y /= num_points;
		average_coordinate.z /= num_points;

		printf("Average is (%f %f %f)\n", average_coordinate.x, average_coordinate.y, average_coordinate.z);

		HC_Open_Segment("average-coordinate");{
			HC_Insert_Marker(average_coordinate.x, average_coordinate.y, average_coordinate.z);
			HC_Set_Marker_Size(3);
			HC_Set_Marker_Symbol("(x)");
			HC_Set_Color("markers = red");
			HC_Set_Visibility("markers = on");
		}HC_Close_Segment();
	}HC_Close_Segment();

	update(view, env);

	HC_Define_Alias("?Picture", view.GetDriverPath());
	HC_Pause();

	return ExitCode::Success;
}





